@@ -49,11 +49,27 @@ ARGUMENT_RENAMES = {
     'route53.update-traffic-policy-comment.version': 'traffic-policy-version',
     'gamelift.create-build.version': 'build-version',
     'gamelift.update-build.version': 'build-version',
+    'gamelift.create-script.version': 'script-version',
+    'gamelift.update-script.version': 'script-version',
     'route53domains.view-billing.start': 'start-time',
     'route53domains.view-billing.end': 'end-time',
     'apigateway.create-rest-api.version': 'api-version',
+    'apigatewayv2.create-api.version': 'api-version',
+    'apigatewayv2.update-api.version': 'api-version',
     'pinpoint.get-campaign-version.version': 'campaign-version',
     'pinpoint.get-segment-version.version': 'segment-version',
+    'pinpoint.delete-email-template.version': 'template-version',
+    'pinpoint.delete-push-template.version': 'template-version',
+    'pinpoint.delete-sms-template.version': 'template-version',
+    'pinpoint.delete-voice-template.version': 'template-version',
+    'pinpoint.get-email-template.version': 'template-version',
+    'pinpoint.get-push-template.version': 'template-version',
+    'pinpoint.get-sms-template.version': 'template-version',
+    'pinpoint.get-voice-template.version': 'template-version',
+    'pinpoint.update-email-template.version': 'template-version',
+    'pinpoint.update-push-template.version': 'template-version',
+    'pinpoint.update-sms-template.version': 'template-version',
+    'pinpoint.update-voice-template.version': 'template-version',
     'stepfunctions.send-task-success.output': 'task-output',
     'clouddirectory.publish-schema.version': 'schema-version',
     'mturk.list-qualification-types.query': 'types-query',
@@ -69,9 +85,19 @@ ARGUMENT_RENAMES = {
     'lex-models.delete-intent-version.version': 'intent-version',
     'lex-models.delete-slot-type-version.version': 'slot-type-version',
     'lex-models.get-export.version': 'resource-version',
+    'license-manager.get-grant.version': 'grant-version',
+    'license-manager.delete-grant.version': 'grant-version',
+    'license-manager.get-license.version': 'license-version',
     'mobile.create-project.region': 'project-region',
     'rekognition.create-stream-processor.output': 'stream-processor-output',
     'eks.create-cluster.version': 'kubernetes-version',
+    'eks.update-cluster-version.version': 'kubernetes-version',
+    'eks.create-nodegroup.version': 'kubernetes-version',
+    'eks.update-nodegroup-version.version': 'kubernetes-version',
+    'schemas.*.version': 'schema-version',
+    'sagemaker.delete-image-version.version': 'version-number',
+    'sagemaker.describe-image-version.version': 'version-number',
+    'iotwireless.*.lo-ra-wan': 'lorawan',
 }
 
 # Same format as ARGUMENT_RENAMES, but instead of renaming the arguments,
